@@ -3,11 +3,14 @@ our $AUTHORITY = 'cpan:GENE';
 
 # ABSTRACT: Glorified metronome
 
-our $VERSION = '0.3003';
+our $VERSION = '0.4000';
 
+use Data::Dumper::Compact qw(ddc);
+use List::Util qw(sum0);
 use Math::Bezier ();
 use MIDI::Util qw(dura_size reverse_dump set_chan_patch set_time_signature);
 use Music::Duration;
+use Music::RhythmSet::Util qw(upsize);
 
 use Moo;
 use strictures 2;
@@ -35,6 +38,7 @@ sub BUILD {
 }
 
 
+has verbose   => ( is => 'ro', default => sub { 0 } );
 has kit       => ( is => 'ro', default => sub { 0 } );
 has reverb    => ( is => 'ro', default => sub { 15 } );
 has channel   => ( is => 'ro', default => sub { 9 } );
@@ -440,6 +444,71 @@ sub sync_patterns {
 }
 
 
+sub add_fill {
+    my ($self, $fill, %patterns) = @_;
+
+    $fill ||= sub {
+        return {
+            duration       => 8,
+            $self->open_hh => '000',
+            $self->snare   => '111',
+            $self->kick    => '000',
+        };
+    };
+    my $fill_patterns = $fill->($self);
+    print 'Fills: ', ddc($fill_patterns) if $self->verbose;
+    my $fill_duration = delete $fill_patterns->{duration} || 8;
+    my $fill_length   = length((values %$fill_patterns)[0]);
+
+    my %lengths;
+    for my $instrument (keys %patterns) {
+        $lengths{$instrument} = sum0 map { length $_ } @{ $patterns{$instrument} };
+    }
+
+    my $lcm = _multilcm($fill_duration, values %lengths);
+    print "LCM: $lcm\n" if $self->verbose;
+
+    my $fill_chop = $fill_duration == $lcm
+        ? $fill_length
+        : int $lcm / $fill_length + 1;
+    print "FC: $fill_chop\n" if $self->verbose;
+
+    my %fresh_patterns;
+    for my $instrument (keys %patterns) {
+        # get a single "flattened" pattern as an arrayref
+        my $pattern = [ map { split //, $_ } @{ $patterns{$instrument} } ];
+        # the fresh pattern is upsized with the LCM
+        $fresh_patterns{$instrument} = @$pattern < $lcm
+            ? [ join '', @{ upsize($pattern, $lcm) } ]
+            : [ join '', @$pattern ];
+    }
+    print 'Patterns: ', ddc(\%fresh_patterns) if $self->verbose;
+
+    my %replacement;
+    for my $instrument (keys %$fill_patterns) {
+        my $pattern = [ split //, sprintf '%0*s', $fill_duration, $fill_patterns->{$instrument} ];
+        my $fresh;
+        if (@$pattern < $lcm) {
+            $fresh = join '', @{ upsize($pattern, $lcm) };
+        }
+        else {
+            $fresh = join '', @$pattern;
+        }
+        $replacement{$instrument} = substr $fresh, -$fill_chop;
+    }
+    print 'Replacements: ', ddc(\%replacement) if $self->verbose;
+
+    for my $instrument (keys %fresh_patterns) {
+        my $string = join '', @{ $fresh_patterns{$instrument} };
+        my $pos = length $replacement{$instrument};
+        substr $string, -$pos, $pos, $replacement{$instrument};
+        print "$instrument: $string\n" if $self->verbose;
+    }
+
+    $self->sync_patterns(%fresh_patterns);
+}
+
+
 sub set_time_sig {
     my ($self, $signature, $set) = @_;
     $self->signature($signature) if $signature;
@@ -464,6 +533,22 @@ sub write {
     $self->score->write_score( $self->file );
 }
 
+sub _gcf {
+    my ($x, $y) = @_;
+    ($x, $y) = ($y, $x % $y) while $y;
+    return $x;
+}
+
+sub _lcm {
+    return($_[0] * $_[1] / _gcf($_[0], $_[1]));
+}
+
+sub _multilcm {
+    my $x = shift;
+    $x = _lcm($x, shift) while @_;
+    return $x;
+}
+
 1;
 
 __END__
@@ -478,7 +563,7 @@ MIDI::Drummer::Tiny - Glorified metronome
 
 =head1 VERSION
 
-version 0.3003
+version 0.4000
 
 =head1 SYNOPSIS
 
@@ -536,6 +621,10 @@ eighth or quarter note, for instance.
 
 =head1 ATTRIBUTES
 
+=head2 verbose
+
+Default: C<0>
+
 =head2 file
 
 Default: C<MIDI-Drummer.mid>
@@ -863,9 +952,28 @@ Defaults:
 
 Execute the C<pattern> method for multiple voices.
 
-Option defaults:
+=head2 add_fill
 
-  duration: quarter-note
+  $d->add_fill( $fill, $instrument1 => $patterns1, $inst2 => $pats2, ... );
+  $d->add_fill(
+      sub {
+          my $self = shift;
+          return {
+            duration       => 16, # sixteenth note fill
+            $self->open_hh => '00000000',
+            $self->snare   => '11111111',
+            $self->kick    => '00000000',
+          };
+      },
+      $d->open_hh => [ ('11111111') x $d->bars ],  # example phrase
+      $d->snare   => [ ('0101') x $d->bars ],      # "
+      $d->kick    => [ ('1010') x $d->bars ],      # "
+  );
+
+Add a fill to the beat pattern.  That is, replace the end of the given
+beat-string phrase with a fill.  The fill is given as the first
+argument and should be a coderef that returns a hashref.  The default
+is a three-note, eighth-note snare fill.
 
 =head2 set_time_sig
 
@@ -896,8 +1004,8 @@ the file name.
 
 =head1 SEE ALSO
 
-The F<eg/*> programs in this distribution. Also
-F<eg/drum-fills-advanced> in the L<Music::Duration::Partition>
+The F<t/*> test file and the F<eg/*> programs in this distribution.
+Also F<eg/drum-fills-advanced> in the L<Music::Duration::Partition>
 distribution.
 
 L<Math::Bezier>
@@ -908,6 +1016,8 @@ L<Moo>
 
 L<Music::Duration>
 
+L<Music::RhythmSet::Util>
+
 L<https://en.wikipedia.org/wiki/General_MIDI#Percussion>
 
 L<https://en.wikipedia.org/wiki/General_MIDI_Level_2#Drum_sounds>
